import logging

from flask import Flask, g, request, abort, flash, render_template, Response, redirect, url_for
from flaskext.mysql import MySQL
from flask_httpauth import HTTPBasicAuth
from passlib import pwd
from passlib.hash import argon2

log = logging.getLogger(__name__)
app = Flask(__name__, template_folder='./templates')
mysql = MySQL(app)
auth = HTTPBasicAuth()

# MySQL configurations
app.config['MYSQL_DATABASE_USER'] = 'project1-web'
app.config['MYSQL_DATABASE_PASSWORD'] = '20dollarsnieceflower29'
app.config['MYSQL_DATABASE_DB'] = 'project1'
app.config['MYSQL_DATABASE_HOST'] = 'localhost'

# session config
app.secret_key = pwd.genword(entropy=128)


def get_data(sql, params=None):
    conn = mysql.connect()
    cursor = conn.cursor()
    records = []

    try:
        log.debug(sql)
        cursor.execute(sql, params)
        result = cursor.fetchall()
        for row in result:
            records.append(list(row))

    except Exception as e:
        log.exception("Fout bij het ophalen van data: {0})".format(e))

    cursor.close()
    conn.close()

    return records


def set_data(sql, params=None):
    conn = mysql.connect()
    cursor = conn.cursor()

    try:
        log.debug(sql)
        cursor.execute(sql, params)
        conn.commit()
        log.debug("SQL uitgevoerd")

    except Exception as e:
        log.exception("Fout bij uitvoeren van sql: {0})".format(e))
        return False

    cursor.close()
    conn.close()

    return True


def activate_pump():
    set_data("UPDATE sensorsActuators SET actuatorState=1 WHERE sensorType='pump';")
    return True


def get_temp():
    try:
        temp = get_data('SELECT value FROM readings WHERE sensorID=1 ORDER BY readingDate DESC LIMIT 1;')
        return temp[0][0]

    except Exception as e:
        message = 'Error fetching temperature data'
        log.error(message)
        return False, message


def get_moisture():
    try:
        dryness = get_data('SELECT value FROM readings WHERE sensorID=2 ORDER BY readingDate DESC LIMIT 1;')
        dryness_value = dryness[0][0]
        if dryness_value >= 35:
            return 'dry'
        elif dryness_value <= 18:
            return 'oversaturated'
        else:
            return 'optimal'

    except Exception as e:
        message = 'Error fetching moisture data'
        log.error(message)
        return False, message


def get_fan_status():
    try:
        fanStatus = get_data('SELECT actuatorState FROM sensorsActuators WHERE sensorType="Fan";')
        rawFanStatus = fanStatus[0][0]
        if rawFanStatus == 0:
            return 'OFF'
        elif rawFanStatus == 1:
            return 'ON'

    except Exception as e:
        message = 'Error fan status'
        log.error(message)
        return False, message


@app.route('/helloworld')
def hello_world():
    return ("<h1>Hello World<h1>")


@app.route('/')
def index():
    current_temperature = get_temp()
    current_moisture = get_moisture()
    current_fanstatus = get_fan_status()
    return render_template('index.html',
                           currentTemperature=current_temperature,
                           currentMoisture=current_moisture,
                           fanStatus=current_fanstatus)


@app.route('/about')
def about():
    return render_template('about.html')


@app.route("/watering/", methods=['POST'])
def watering():
    # Watering code
    activate_pump()
    forward_message = "Watering..."
    return render_template('index.html', message=forward_message)


# @app.route("/fantoggle/", methods=['POST'])
# def fan_toggle():
#     # Fan toggle code
#     forward_message = "Toggling fan"
#     return render_template('index.html', message=forward_message)


@app.route('/json')
def json():
    from flask import jsonify
    current_temperature = get_temp()
    current_moisture = get_moisture()
    current_fanstatus = get_fan_status()
    return jsonify(temp=current_temperature,
                   moist=current_moisture,
                   fan=current_fanstatus)


# @app.route('/secure')
# @auth.login_required
# def secure():
#     return 'Hello, {}!'.format(g.user)


# @app.route('/register', methods=['GET', 'POST'])
# def register():
#     result = None
#     if request.method == 'POST':
#         user = request.form.get('user')
#         password = request.form.get('password')
#         if not user and password:
#             abort(400)
#         result, message = add_user(user, password)
#         flash(message)
#     return render_template('register.html', success=result)


if __name__ == '__main__':
    logging.basicConfig(level=logging.DEBUG)
    log.info("Flask app starting")
    app.run(host='0.0.0.0', debug=True)
